@@ -241,15 +241,13 @@ skip_unlock_exit:
 SQLRETURN insert_params(STMT *stmt, SQLULEN row, char **finalquery,
                         SQLULEN *finalquery_length)
 {
-  char *query= GET_QUERY(&stmt->query), *to;
+  char *query= GET_QUERY(&stmt->query);
   uint i,length, had_info= 0;
-  NET *net;
   SQLRETURN rc= SQL_SUCCESS;
 
   int mutex_was_locked= myodbc_mutex_trylock(&stmt->dbc->lock);
 
-  net= &stmt->dbc->mysql.net;
-  to= (char*) net->buff + (finalquery_length!= NULL ? *finalquery_length : 0);
+  //to = stmt->buf() + (finalquery_length != NULL ? *finalquery_length : 0);
 
   if (!stmt->dbc->ds->dont_use_set_locale)
   {
@@ -283,21 +281,21 @@ SQLRETURN insert_params(STMT *stmt, SQLULEN row, char **finalquery,
     {
       bind= get_param_bind(stmt, i, TRUE);
 
-      rc= insert_param(stmt, (uchar*)bind, stmt->apd, aprec, iprec, row);
+      rc = insert_param(stmt, bind, stmt->apd, aprec, iprec, row);
     }
     else
     {
       pos= get_param_pos(&stmt->query, i);
       length= (uint) (pos-query);
 
-      if ( !(to= add_to_buffer(net,to,query,length)) )
+      if (stmt->add_to_buffer(query, length) == NULL)
       {
         goto memerror;
       }
 
       query= pos+1;  /* Skip '?' */
 
-      rc= insert_param(stmt, (uchar*)&to, stmt->apd, aprec, iprec, row);
+      rc= insert_param(stmt, NULL, stmt->apd, aprec, iprec, row);
     }
 
     if (!SQL_SUCCEEDED(rc))
@@ -325,28 +323,25 @@ SQLRETURN insert_params(STMT *stmt, SQLULEN row, char **finalquery,
   {
     length= (uint) (GET_QUERY_END(&stmt->query) - query);
 
-    if ( !(to= add_to_buffer(net, to, query, length + 1)) )
+    if (stmt->add_to_buffer(query, length) == NULL)
     {
       goto memerror;
     }
 
     if (finalquery_length!= NULL)
     {
-      *finalquery_length= to - (char*)net->buff - 1;
+      *finalquery_length = stmt->buf_pos();
     }
 
     if (finalquery!=NULL)
     {
-      if ( !(to= (char*) myodbc_memdup((char*) net->buff,
-        (uint) (to - (char*) net->buff),MYF(0))) )
+      char *dupquery = NULL;
+      if (!(dupquery = (char*)myodbc_memdup(stmt->buf(),
+        stmt->buf_pos(), MYF(0))))
       {
         goto memerror;
       }
-    }
-
-    if (finalquery!=NULL)
-    {
-      *finalquery= to;
+      *finalquery= dupquery;
     }
   }
 
@@ -376,30 +371,30 @@ error:
 }
 
 static
-void put_null_param(STMT *stmt, NET* net, char** toptr, MYSQL_BIND *bind)
+void put_null_param(STMT *stmt, MYSQL_BIND *bind)
 {
-  if (ssps_used(stmt))
+  if (bind != NULL && ssps_used(stmt))
   {
     bind->is_null_value= '\1';
   }
   else
   {
-    *toptr= add_to_buffer(net, *toptr, "NULL", 4);
+    stmt->add_to_buffer("NULL", 4);
   }
 }
 
 
 static
-void put_default_value(STMT *stmt, NET* net, char** toptr, MYSQL_BIND *bind)
+void put_default_value(STMT *stmt, MYSQL_BIND *bind)
 {
-  if (ssps_used(stmt))
+  if (bind != NULL && ssps_used(stmt))
   {
     /* That is actually wrong, but i can't see a good way */
     bind->is_null_value= '\1';
   }
   else
   {
-    *toptr= add_to_buffer(net, *toptr, "DEFAULT", 7);
+    stmt->add_to_buffer("DEFAULT", 7);
   }
 }
 
@@ -407,6 +402,9 @@ void put_default_value(STMT *stmt, NET* net, char** toptr, MYSQL_BIND *bind)
 static
 BOOL allocate_param_buffer(MYSQL_BIND *bind, unsigned long length)
 {
+  if (bind == NULL)
+    return FALSE;
+
   /* have to be very careful with that. it is probably better to put into
        a separate data structure. and free right after use */
   if (bind->buffer == NULL)
@@ -464,7 +462,7 @@ BOOL bind_param(MYSQL_BIND *bind, const char *value, unsigned long length,
 
 /* TRUE - on memory allocation error */
 static
-BOOL put_param_value(STMT *stmt, NET* net, char** toptr, MYSQL_BIND *bind,
+BOOL put_param_value(STMT *stmt, MYSQL_BIND *bind,
                      const char * value, unsigned long length)
 {
   if (ssps_used(stmt))
@@ -473,7 +471,7 @@ BOOL put_param_value(STMT *stmt, NET* net, char** toptr, MYSQL_BIND *bind,
   }
   else
   {
-    *toptr= add_to_buffer(net, *toptr, value, length);
+    stmt->add_to_buffer(value, length);
   }
 
   return FALSE;
@@ -772,22 +770,17 @@ SQLRETURN convert_c_type2str(STMT *stmt, SQLSMALLINT ctype, DESCREC *iprec,
   @param[in]      aprec The APD record of the parameter
   @param[in]      iprec The IPD record of the parameter
 */
-SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
+SQLRETURN insert_param(STMT *stmt, MYSQL_BIND *bind, DESC* apd,
                        DESCREC *aprec, DESCREC *iprec, SQLULEN row)
 {
     long length;
     char buff[128], *data= NULL;
     BOOL convert= FALSE, free_data= FALSE;
     DBC *dbc= stmt->dbc;
-    NET *net= &dbc->mysql.net;
     SQLLEN *octet_length_ptr= NULL;
     SQLLEN *indicator_ptr= NULL;
     SQLRETURN result= SQL_SUCCESS;
 
-    /* carefully should be used either these 2 or that(bind) ptr. union? */
-    char **toptr= (char**)place4param, *to= *toptr;
-    MYSQL_BIND * bind= (MYSQL_BIND*)place4param;
-
     if (aprec->octet_length_ptr)
     {
       octet_length_ptr= (SQLLEN*)ptr_offset_adjust(aprec->octet_length_ptr,
@@ -812,7 +805,7 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
 
     if (indicator_ptr && *indicator_ptr == SQL_NULL_DATA)
     {
-      put_null_param(stmt, net, toptr, bind);
+      put_null_param(stmt, bind);
 
       return SQL_SUCCESS;
     }
@@ -861,7 +854,7 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
               aprec->concise_type == SQL_C_DEFAULT &&
               aprec->par.value == NULL))
     {
-      put_default_value(stmt, net, toptr, bind);
+      put_default_value(stmt, bind);
       return SQL_SUCCESS;
     }
     else if (IS_DATA_AT_EXEC(octet_length_ptr))
@@ -869,7 +862,7 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
         length= aprec->par.value_length;
         if ( !(data= aprec->par.value) )
         {
-          put_default_value(stmt, net, toptr, bind);
+          put_default_value(stmt, bind);
           return SQL_SUCCESS;
         }
     }
@@ -914,7 +907,7 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
         if (data[0] == '{')       /* Of type {d date } */
         {
           /* TODO: check if we need to check for truncation here as well? */
-          if (ssps_used(stmt))
+          if (bind != NULL && ssps_used(stmt))
           {
             if (bind_param(bind, data, length,
                         map_sql2mysql_type(iprec->concise_type)))
@@ -924,7 +917,7 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
           }
           else
           {
-            to= add_to_buffer(net, to, data, length);
+            stmt->add_to_buffer(data, length);
           }
           goto out;
         }
@@ -962,7 +955,7 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
       case SQL_VARBINARY:
       case SQL_LONGVARBINARY:
         {
-          if (ssps_used(stmt))
+          if (bind != NULL && ssps_used(stmt))
           {
             /* i guess for ssps we do not need introducer */
             convert= 0;
@@ -972,7 +965,7 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
             if (dbc->cxn_charset_info->number !=
                 dbc->ansi_charset_info->number)
             {
-              to= add_to_buffer(net, to, "_binary", 7);
+              stmt->add_to_buffer("_binary", 7);
             }
           }
           /* We have only added the introducer, data is added below. */
@@ -989,7 +982,7 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
           if (aprec->concise_type == SQL_C_WCHAR &&
               dbc->cxn_charset_info->number != UTF8_CHARSET_NUMBER)
           {
-            if (ssps_used(stmt))
+            if (bind != NULL && ssps_used(stmt))
             {
                 if (bind_param(bind, data, length, MYSQL_TYPE_BLOB))
               {
@@ -1000,14 +993,14 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
             }
             else
             {
-              to= add_to_buffer(net, to, "_utf8", 5);
+              stmt->add_to_buffer("_utf8", 5);
             }
           }
           else if (aprec->concise_type != SQL_C_WCHAR &&
                     dbc->cxn_charset_info->number !=
                     dbc->ansi_charset_info->number)
           {
-            if (ssps_used(stmt))
+            if (bind != NULL && ssps_used(stmt))
             {
               if (bind_param(bind, data, length, MYSQL_TYPE_BLOB))
               {
@@ -1018,9 +1011,9 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
             }
             else
             {
-              to= add_to_buffer(net, to, "_", 1);
-              to= add_to_buffer(net, to, dbc->ansi_charset_info->csname,
-                              strlen(dbc->ansi_charset_info->csname));
+              stmt->add_to_buffer("_", 1);
+              stmt->add_to_buffer(dbc->ansi_charset_info->csname,
+                                  strlen(dbc->ansi_charset_info->csname));
             }
           }
           /* We have only added the introducer, data is added below. */
@@ -1036,7 +1029,7 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
         }
         else
         {
-          if (put_param_value(stmt, net, &to, bind, buff, length))
+          if (put_param_value(stmt, bind, buff, length))
           {
             goto memerror;
           }
@@ -1062,7 +1055,7 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
             return set_stmt_error(stmt, "22008", "Fractional truncation", 0);
           }
 
-          if (ssps_used(stmt))
+          if (bind != NULL && ssps_used(stmt))
           {
             length= sprintf(buff, "%02d:%02d:%02d", time->hour, time->minute, time->second);
           }
@@ -1071,7 +1064,7 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
             length= sprintf(buff, "'%02d:%02d:%02d'", time->hour, time->minute, time->second);
           }
 
-          if (put_param_value(stmt, net, &to, bind, buff, length))
+          if (put_param_value(stmt, bind, buff, length))
           {
             goto memerror;
           }
@@ -1100,7 +1093,7 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
             return set_stmt_error(stmt, "22008", "Not a valid time value supplied", 0);
           }
 
-          if (ssps_used(stmt))
+          if (bind != NULL && ssps_used(stmt))
           {
             length= sprintf(buff, "%02d:%02d:%02d",
                   hours,
@@ -1115,7 +1108,7 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
                   (int) time%100);
           }
 
-          if (put_param_value(stmt, net, &to, bind, buff, length))
+          if (put_param_value(stmt, bind, buff, length))
           {
             goto memerror;
           }
@@ -1125,7 +1118,7 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
 
       case SQL_BIT:
         {
-          if (ssps_used(stmt))
+          if (bind != NULL && ssps_used(stmt))
           {
             char bit_val= atoi(data)!= 0 ? 1 : 0;
             /* Generic ODBC supports only BIT(1) */
@@ -1133,7 +1126,7 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
           }
           else if (!convert)
           {
-            to= add_to_buffer(net, to, data, 1);
+            stmt->add_to_buffer(data, 1);
           }
           goto out;
         }
@@ -1189,12 +1182,12 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
       default:
         if (!convert)
         {
-          put_param_value(stmt, net, &to, bind, data, length);
+          put_param_value(stmt, bind, data, length);
           goto out;
         }
     }
 
-    if (ssps_used(stmt))
+    if (bind != NULL && ssps_used(stmt))
     {
       bind_param(bind, data, length, MYSQL_TYPE_STRING);
     }
@@ -1205,27 +1198,28 @@ SQLRETURN insert_param(STMT *stmt, uchar *place4param, DESC* apd,
        is_binary_sql_type(iprec->concise_type))
       {
         SQLLEN transformed_len = 0;
-        to= add_to_buffer(net, to, " 0x", 3);
+        stmt->add_to_buffer(" 0x", 3);
         /* Make sure we have room for a fully-escaped string. */
-        if (!(to= extend_buffer(net, to, length * 2)))
+        if (!stmt->extend_buffer(length * 2))
         {
           goto memerror;
         }
 
-        copy_binhex_result(stmt, (SQLCHAR*)to, length * 2 + 1, &transformed_len, 0, data, length);
-        to += transformed_len;
+        copy_binhex_result(stmt, (SQLCHAR*)stmt->endbuf(), length * 2 + 1, &transformed_len, 0, data, length);
+        stmt->buf_add_pos(transformed_len);
       }
       else
       {
-        to= add_to_buffer(net,to,"'",1);
+        stmt->add_to_buffer("'", 1);
         /* Make sure we have room for a fully-escaped string. */
-        if ( !(to= extend_buffer(net, to, length * 2)) )
+        if (!(stmt->extend_buffer(length * 2)))
         {
           goto memerror;
         }
 
-        to+= mysql_real_escape_string(&dbc->mysql, to, data, length);
-        to= add_to_buffer(net, to, "'", 1);
+        size_t added = mysql_real_escape_string(&dbc->mysql, stmt->endbuf(), data, length);
+        stmt->buf_add_pos(added);
+        stmt->add_to_buffer("'", 1);
       }
     }
 
@@ -1235,8 +1229,6 @@ out:
       x_free(data);
     }
 
-    *toptr= to;
-
     return result;
 
 memerror:
@@ -1529,8 +1521,7 @@ SQLRETURN my_SQLExecute( STMT *pStmt )
           const char * stmtsBinder= " UNION ALL ";
           const ulong binderLength= strlen(stmtsBinder);
 
-          add_to_buffer(&pStmt->dbc->mysql.net, (char*)pStmt->dbc->mysql.net.buff + length,
-                     stmtsBinder, binderLength);
+          pStmt->add_to_buffer(stmtsBinder, binderLength);
           length+= binderLength;
         }
         else
