@@ -1,52 +1,116 @@
-/*
-  Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
-
-  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-  MySQL Connectors. There are special exceptions to the terms and
-  conditions of the GPLv2 as it is applied to this software, see the
-  FLOSS License Exception
-  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-  
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; version 2 of the License.
-  
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-  for more details.
-  
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 #include "odbctap.h"
 
+DECLARE_TEST(t_bug_28258500)
+{
+  char err_msg[256] = { 0 };
+  SQLCHAR sql_state[6] = { 0 };
+  SQLINTEGER native_err = 0;
+  SQLSMALLINT msg_len = 0;
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+  alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, NULL, NULL, NULL,
+                               "MULTI_STATEMENTS=1");
+
+  ok_sql(hstmt1, "SET @Temp=1;CALL NonExistentProc()");
+  expect_stmt(hstmt1, SQLMoreResults(hstmt1), SQL_ERROR);
+  ok_stmt(hstmt1, SQLGetDiagRec(SQL_HANDLE_STMT, hstmt1, 1, sql_state,
+                               &native_err, (SQLCHAR*)err_msg,
+                               sizeof(err_msg), &msg_len ));
+  is(strstr(err_msg, "NonExistentProc") != NULL);
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+  return OK;
+}
+
+DECLARE_TEST(t_get_diag_all)
+{
+  SQLCHAR buf[1024];
+  SQLSMALLINT str_len_ptr = 0;
+  SQLLEN data1 = 0;
+  SQLINTEGER data2 = 0;
+  SQLRETURN data3 = 0;
+
+  expect_sql(hstmt, "This is an invalid Query! (odbc test)", SQL_ERROR);
+  printf("** Start SQLGetDiagField!\n");
+
+  SQLGetDiagField(SQL_HANDLE_STMT, hstmt, 1, SQL_DIAG_CURSOR_ROW_COUNT, &data1,
+                  sizeof(data1), &str_len_ptr);
+  SQLGetDiagField(SQL_HANDLE_STMT, hstmt, 1, SQL_DIAG_DYNAMIC_FUNCTION, buf,
+                  sizeof(buf), &str_len_ptr);
+  SQLGetDiagField(SQL_HANDLE_STMT, hstmt, 1, SQL_DIAG_DYNAMIC_FUNCTION_CODE, &data2,
+                  sizeof(data2), &str_len_ptr);
+  SQLGetDiagField(SQL_HANDLE_STMT, hstmt, 1, SQL_DIAG_NUMBER, &data2,
+                  sizeof(data2), &str_len_ptr);
+  SQLGetDiagField(SQL_HANDLE_STMT, hstmt, 1, SQL_DIAG_RETURNCODE, &data3,
+                  sizeof(data3), &str_len_ptr);
+  SQLGetDiagField(SQL_HANDLE_STMT, hstmt, 1, SQL_DIAG_ROW_COUNT, &data1,
+                  sizeof(data1), &str_len_ptr);
+  SQLGetDiagField(SQL_HANDLE_STMT, hstmt, 1, SQL_DIAG_CLASS_ORIGIN, buf,
+                  sizeof(buf), &str_len_ptr);
+  SQLGetDiagField(SQL_HANDLE_STMT, hstmt, 1, SQL_DIAG_COLUMN_NUMBER, &data2,
+                  sizeof(data2), &str_len_ptr);
+  SQLGetDiagField(SQL_HANDLE_STMT, hstmt, 1, SQL_DIAG_CONNECTION_NAME, buf,
+                  sizeof(buf), &str_len_ptr);
+  SQLGetDiagField(SQL_HANDLE_STMT, hstmt, 1, SQL_DIAG_MESSAGE_TEXT, buf,
+                  sizeof(buf), &str_len_ptr);
+  SQLGetDiagField(SQL_HANDLE_STMT, hstmt, 1, SQL_DIAG_ROW_NUMBER, &data1,
+                  sizeof(data1), &str_len_ptr);
+  SQLGetDiagField(SQL_HANDLE_STMT, hstmt, 1, SQL_DIAG_SERVER_NAME, buf,
+                  sizeof(buf), &str_len_ptr);
+  SQLGetDiagField(SQL_HANDLE_STMT, hstmt, 1, SQL_DIAG_SQLSTATE, buf,
+                  sizeof(buf), &str_len_ptr);
+  SQLGetDiagField(SQL_HANDLE_STMT, hstmt, 1, SQL_DIAG_SUBCLASS_ORIGIN, buf,
+                  sizeof(buf), &str_len_ptr);
+
+  return OK;
+}
+
 
 DECLARE_TEST(t_odbc3_error)
 {
-  SQLHENV henv1;
-  SQLHDBC hdbc1;
-  SQLHSTMT hstmt1;
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
   SQLINTEGER ov_version;
+  /*SQLCHAR dm_version[6];*/
 
-  ok_env(henv1, SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv1));
-  ok_env(henv1, SQLSetEnvAttr(henv1, SQL_ATTR_ODBC_VERSION,
-                              (SQLPOINTER)SQL_OV_ODBC3, 0));
+  is(OK == alloc_basic_handles(&henv1, &hdbc1, &hstmt1));
 
-  ok_env(henv1, SQLAllocHandle(SQL_HANDLE_DBC, henv1, &hdbc1));
+  /*SQLGetInfo(hdbc1, SQL_DM_VER, (SQLPOINTER)dm_version, 6, 0);*/
 
   ok_env(henv1, SQLGetEnvAttr(henv1, SQL_ATTR_ODBC_VERSION,
                               (SQLPOINTER)&ov_version, 0, 0));
+  /* "For standards-compliant applications, SQLAllocHandle is mapped to SQLAllocHandleStd
+     at compile time. The difference between these two functions is that SQLAllocHandleStd
+     sets the SQL_ATTR_ODBC_VERSION environment attribute to SQL_OV_ODBC3 when it is called
+     with the HandleType argument set to SQL_HANDLE_ENV. This is done because standards-compliant
+     applications are always ODBC 3.x applications." */
   is_num(ov_version, SQL_OV_ODBC3);
 
-  ok_con(hdbc1, SQLConnect(hdbc1, mydsn, SQL_NTS, myuid, SQL_NTS,
-                           mypwd, SQL_NTS));
-
-  ok_con(hdbc1, SQLAllocHandle(SQL_HANDLE_STMT, hdbc1, &hstmt1));
-
   expect_sql(hstmt1, "SELECT * FROM non_existing_table", SQL_ERROR);
   if (check_sqlstate(hstmt1, "42S02") != OK)
     return FAIL;
@@ -60,21 +124,8 @@ DECLARE_TEST(t_odbc3_error)
 
   ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
 
-  expect_stmt(hstmt1, SQLSetStmtAttr(hstmt1, SQL_ATTR_FETCH_BOOKMARK_PTR,
-                                     (SQLPOINTER)NULL, 0),
-              SQL_ERROR);
-  if (check_sqlstate(hstmt1, "HYC00") != OK)
-    return FAIL;
-
-  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
-
   ok_sql(hstmt1, "DROP TABLE IF EXISTS t_error");
-
-  ok_con(hdbc1, SQLDisconnect(hdbc1));
-
-  ok_con(hdbc1, SQLFreeHandle(SQL_HANDLE_DBC, hdbc1));
-
-  ok_env(henv1, SQLFreeHandle(SQL_HANDLE_ENV, henv1));
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
 
   return OK;
 }
@@ -82,9 +133,7 @@ DECLARE_TEST(t_odbc3_error)
 
 DECLARE_TEST(t_odbc2_error)
 {
-  SQLHENV henv1;
-  SQLHDBC hdbc1;
-  SQLHSTMT hstmt1;
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
   SQLINTEGER ov_version;
 
   ok_env(henv1, SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv1));
@@ -125,16 +174,37 @@ DECLARE_TEST(t_odbc2_error)
 
   ok_sql(hstmt1, "DROP TABLE IF EXISTS t_error");
 
-  ok_con(hdbc1, SQLDisconnect(hdbc1));
-
-  ok_con(hdbc1, SQLFreeHandle(SQL_HANDLE_DBC, hdbc1));
-
-  ok_env(henv1, SQLFreeHandle(SQL_HANDLE_ENV, henv1));
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
 
   return OK;
 }
 
 
+#ifndef USE_IODBC
+/* Testing ability of application to set 3.8 ODBC version */
+DECLARE_TEST(t_odbc3_80)
+{
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+  SQLINTEGER ov_version;
+
+  ok_env(henv1, SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv1));
+  /* ODBC 3.8 applications should use ... SQLSetEnvAttr to set the SQL_ATTR_ODBC_VERSION environment
+     attribute to SQL_OV_ODBC_3_80*/
+  ok_env(henv1, SQLSetEnvAttr(henv1, SQL_ATTR_ODBC_VERSION,
+                              (SQLPOINTER)SQL_OV_ODBC3_80, 0));
+
+  ok_env(henv1, SQLAllocHandle(SQL_HANDLE_DBC, henv1, &hdbc1));
+
+  ok_env(henv1, SQLGetEnvAttr(henv1, SQL_ATTR_ODBC_VERSION,
+                              (SQLPOINTER)&ov_version, 0, 0));
+  is_num(ov_version, SQL_OV_ODBC3_80);
+
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+
+  return OK;
+}
+#endif
+
 DECLARE_TEST(t_diagrec)
 {
   SQLCHAR   sqlstate[6]= {0};
@@ -364,7 +434,7 @@ DECLARE_TEST(getdata_need_nullind)
   SQLINTEGER i;
   ok_sql(hstmt, "select 1 as i , null as j ");
   ok_stmt(hstmt, SQLFetch(hstmt));
-  
+
   /* that that nullind ptr is ok when data isn't null */
   ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_LONG, &i, 0, NULL));
 
@@ -460,7 +530,7 @@ DECLARE_TEST(t_bug13542600)
 DECLARE_TEST(t_bug14285620)
 {
   SQLSMALLINT data_type, dec_digits, nullable, cblen;
-  SQLUINTEGER info, col_size; 
+  SQLUINTEGER info, col_size;
   SQLINTEGER timeout= 20, cbilen;
   SQLCHAR szData[255]={0};
 
@@ -468,16 +538,16 @@ DECLARE_TEST(t_bug14285620)
   expect_dbc(hdbc, SQLGetConnectAttr(hdbc, SQL_ATTR_LOGIN_TIMEOUT, NULL, 0, NULL), SQL_SUCCESS);
   expect_dbc(hdbc, SQLGetConnectAttr(hdbc, SQL_ATTR_LOGIN_TIMEOUT, &timeout, 0, NULL), SQL_SUCCESS);
   is_num(timeout, 0);
-  
+
   /* Character attribute */
-  /* 
+  /*
     In this particular case MSSQL always returns SQL_SUCCESS_WITH_INFO
     apparently because the driver manager always provides the buffer even
     when the client application passes NULL
   */
 #ifdef _WIN32
-  /* 
-    This check is only relevant to Windows Driver Manager 
+  /*
+    This check is only relevant to Windows Driver Manager
   */
   expect_dbc(hdbc, SQLGetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, NULL, 0, NULL), SQL_SUCCESS_WITH_INFO);
 #endif
@@ -486,10 +556,10 @@ DECLARE_TEST(t_bug14285620)
   /*
   MSDN Says about the last parameter &cblen for SQLGetInfo,
   functions:
-  
-     If InfoValuePtr is NULL, StringLengthPtr will still return the 
-     total number of bytes (excluding the null-termination character 
-     for character data) available to return in the buffer pointed 
+
+     If InfoValuePtr is NULL, StringLengthPtr will still return the
+     total number of bytes (excluding the null-termination character
+     for character data) available to return in the buffer pointed
      to by InfoValuePtr.
 
      http://msdn.microsoft.com/en-us/library/ms710297%28v=vs.85%29
@@ -499,14 +569,14 @@ DECLARE_TEST(t_bug14285620)
   expect_dbc(hdbc, SQLGetInfo(hdbc, SQL_AGGREGATE_FUNCTIONS, &info, 0, &cblen), SQL_SUCCESS);
   is_num(cblen, 4);
 
-  is_num(info, (SQL_AF_ALL | SQL_AF_AVG | SQL_AF_COUNT | SQL_AF_DISTINCT | 
+  is_num(info, (SQL_AF_ALL | SQL_AF_AVG | SQL_AF_COUNT | SQL_AF_DISTINCT |
              SQL_AF_MAX | SQL_AF_MIN | SQL_AF_SUM));
 
   /* Get database name for further checks */
   expect_dbc(hdbc, SQLGetInfo(hdbc, SQL_DATABASE_NAME, szData, sizeof(szData), NULL), SQL_SUCCESS);
   expect_dbc(hdbc, SQLGetInfo(hdbc, SQL_DATABASE_NAME, NULL, 0, &cblen), SQL_SUCCESS);
 
-#ifdef _WIN32  
+#ifdef _WIN32
   /* Windows uses unicode driver by default */
   is_num(cblen, strlen(szData)*sizeof(SQLWCHAR));
 #else
@@ -518,7 +588,7 @@ DECLARE_TEST(t_bug14285620)
   /* Get the native string for further checks */
   expect_dbc(hdbc, SQLNativeSql(hdbc, "SELECT 10", SQL_NTS, szData, sizeof(szData), NULL), SQL_SUCCESS);
   expect_dbc(hdbc, SQLNativeSql(hdbc, "SELECT 10", SQL_NTS, NULL, 0, &cbilen), SQL_SUCCESS);
-  
+
   /* Do like MSSQL, which does calculate as char_count*sizeof(SQLWCHAR) */
   is_num(cbilen, strlen(szData));
 
@@ -527,7 +597,7 @@ DECLARE_TEST(t_bug14285620)
   /* Get the cursor name for further checks */
   expect_stmt(hstmt, SQLGetCursorName(hstmt, szData, sizeof(szData), NULL), SQL_SUCCESS);
   expect_stmt(hstmt, SQLGetCursorName(hstmt, NULL, 0, &cblen), SQL_SUCCESS);
-  
+
   /* Do like MSSQL, which does calculate as char_count*sizeof(SQLWCHAR) */
   is_num(cblen, strlen(szData));
 
@@ -569,6 +639,39 @@ DECLARE_TEST(t_bug14285620)
   return OK;
 }
 
+/*
+  Bug49466: SQLMoreResults does not set statement errors correctly
+  Wrong error message returned from SQLMoreResults
+*/
+DECLARE_TEST(t_bug49466)
+{
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+  SQLCHAR message[SQL_MAX_MESSAGE_LENGTH + 1];
+  SQLCHAR sqlstate[SQL_SQLSTATE_SIZE + 1];
+  SQLINTEGER error;
+  SQLSMALLINT len;
+
+  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
+                                        NULL, NULL, NULL,
+                                        "OPTION=67108864"));//;NO_SSPS=1"));
+
+  ok_stmt(hstmt1, SQLExecDirect(hstmt1, "SELECT 100; CALL t_bug49466proc()", SQL_NTS));
+
+  ok_stmt(hstmt1, SQLFetch(hstmt1));
+  is_num(my_fetch_int(hstmt1, 1), 100);
+
+  SQLMoreResults(hstmt1);
+
+  ok_stmt(hstmt, SQLGetDiagRec(SQL_HANDLE_STMT, hstmt1, 1, sqlstate, &error,
+                               message, sizeof(message), &len));
+  is_num(error, 1305);
+  is(strstr((char *)message, "t_bug49466proc does not exist"));
+
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+
+  return OK;
+}
+
 
 DECLARE_TEST(t_passwordexpire)
 {
@@ -581,16 +684,17 @@ DECLARE_TEST(t_passwordexpire)
   }
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_password_expire");
-  SQLExecDirect(hstmt, (SQLCHAR *)"DROP USER t_pwd_expire", SQL_NTS);
+  SQLExecDirect(hstmt, (SQLCHAR *)"DROP USER IF EXISTS t_pwd_expire", SQL_NTS);
 
-  ok_sql(hstmt, "GRANT ALL ON *.* TO  t_pwd_expire IDENTIFIED BY 'foo'");
+  ok_sql(hstmt, "CREATE USER t_pwd_expire IDENTIFIED BY 'foo'");
+  ok_sql(hstmt, "GRANT ALL ON *.* TO t_pwd_expire");
   ok_sql(hstmt, "ALTER USER t_pwd_expire PASSWORD EXPIRE");
 
   ok_env(henv, SQLAllocConnect(henv, &hdbc1));
 
   /* Expecting error without OPT_CAN_HANDLE_EXPIRED_PASSWORDS */
-  expect_dbc(hdbc1, get_connection(&hdbc1, NULL, "t_pwd_expire", "foo", NULL, NULL),
-              SQL_ERROR);
+  expect_dbc(hdbc1, get_connection(&hdbc1, NULL, "t_pwd_expire", "foo",
+             NULL, NULL), SQL_ERROR);
 
   {
     SQLCHAR sql_state[6];
@@ -602,21 +706,21 @@ DECLARE_TEST(t_passwordexpire)
                   SQL_MAX_MESSAGE_LENGTH - 1, &err_len);
 
     /* ER_MUST_CHANGE_PASSWORD = 1820, ER_MUST_CHANGE_PASSWORD_LOGIN = 1862 */
-    if (strncmp(sql_state, "08004", 5) != 0 || err_code != 1820)
+    if (strncmp(sql_state, "08004", 5) != 0 || !(err_code == 1820 || err_code == 1862))
     {
       printMessage("%s %d %s", sql_state, err_code, err_msg);
-      return FAIL;
+      is(FALSE);
     }
   }
 
   /* Expecting error as password has not been reset */
-  ok_con(hdbc1, get_connection(&hdbc1, NULL, "t_pwd_expire", "foo", NULL,
-                                "CAN_HANDLE_EXP_PWD=1"));
+  ok_con(hdbc1, get_connection(&hdbc1, NULL, "t_pwd_expire", "foo",
+                                NULL, "CAN_HANDLE_EXP_PWD=1"));
 
   /*strcat((char *)conn_in, ";INITSTMT={set password= password('bar')}");*/
   ok_con(hdbc1, SQLAllocStmt(hdbc1, &hstmt1));
 
-  ok_sql(hstmt1, "SET PASSWORD= password('bar')");
+  ok_sql(hstmt1, "SET PASSWORD='bar'");
 
   /* Just to verify that we got normal connection */
   ok_sql(hstmt1, "select 1");
@@ -633,8 +737,7 @@ DECLARE_TEST(t_passwordexpire)
   /* Also verifying that we got normal connection */
   ok_sql(hstmt1, "select 1");
 
-  ok_con(hdbc1, SQLFreeStmt(hstmt1, SQL_DROP));
-
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_DROP));
   ok_con(hdbc1, SQLDisconnect(hdbc1));
   ok_con(hdbc1, SQLFreeConnect(hdbc1));
 
@@ -650,13 +753,10 @@ DECLARE_TEST(t_passwordexpire)
 DECLARE_TEST(t_cleartext_password)
 {
   SQLHDBC hdbc1;
-  SQLHSTMT hstmt1;
-  SQLRETURN rc;
   SQLCHAR sql_state[6];
-  SQLINTEGER  err_code= 0;                              
+  SQLINTEGER  err_code= 0;
   SQLCHAR     err_msg[SQL_MAX_MESSAGE_LENGTH]= {0};
   SQLSMALLINT err_len= 0;
-  SQLCHAR server_version[MYSQL_NAME_LEN+1];
   unsigned int major1= 0, minor1= 0, build1= 0;
 
   if (!mysql_min_version(hdbc, "5.5.16", 6) )
@@ -666,18 +766,18 @@ DECLARE_TEST(t_cleartext_password)
 
   SQLExecDirect(hstmt, (SQLCHAR *)"DROP USER 't_ct_user'@'%'", SQL_NTS);
 
-  if (!SQL_SUCCEEDED(SQLExecDirect(hstmt, 
+  if (!SQL_SUCCEEDED(SQLExecDirect(hstmt,
             "GRANT ALL ON *.* TO "
             "'t_ct_user'@'%' IDENTIFIED WITH "
-            "'authentication_pam'", SQL_NTS))) 
+            "'authentication_pam'", SQL_NTS)))
   {
     skip("The authentication_pam plugin not loaded");
   }
 
   ok_env(henv, SQLAllocConnect(henv, &hdbc1));
 
-  /* 
-    Expecting error CR_AUTH_PLUGIN_CANNOT_LOAD_ERROR 
+  /*
+    Expecting error CR_AUTH_PLUGIN_CANNOT_LOAD_ERROR
     without option ENABLE_CLEARTEXT_PLUGIN
   */
   if(!SQL_SUCCEEDED(get_connection(&hdbc1, mydsn, "t_ct_user", "t_ct_pass",
@@ -688,13 +788,13 @@ DECLARE_TEST(t_cleartext_password)
 
     printMessage("%s %d %s", sql_state, err_code, err_msg);
     if ((strncmp(sql_state, "08004", 5) != 0 || err_code != 2059))
-    {                                                                               
+    {
       return FAIL;
     }
-  }  
+  }
 
-  /* 
-    Expecting error other then CR_AUTH_PLUGIN_CANNOT_LOAD_ERROR 
+  /*
+    Expecting error other then CR_AUTH_PLUGIN_CANNOT_LOAD_ERROR
     as option ENABLE_CLEARTEXT_PLUGIN is used
   */
   if(!SQL_SUCCEEDED(get_connection(&hdbc1, mydsn, "t_ct_user", "t_ct_pass",
@@ -705,7 +805,7 @@ DECLARE_TEST(t_cleartext_password)
     printMessage("%s %d %s", sql_state, err_code, err_msg);
 
     if ((strncmp(sql_state, "08004", 5) == 0 && err_code == 2059))
-    {                                                                               
+    {
       return FAIL;
     }
   }
@@ -716,28 +816,102 @@ DECLARE_TEST(t_cleartext_password)
 }
 
 
+DECLARE_TEST(t_bug11750296)
+{
+  SQLINTEGER rowCount;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS bug11750296");
+  ok_sql(hstmt, "CREATE TABLE bug11750296(Col Integer)");
+  ok_sql(hstmt, "INSERT INTO bug11750296 VALUES(1),(2),(3)");
+
+  ok_stmt(hstmt, SQLGetDiagField(SQL_HANDLE_STMT, hstmt, 0,
+                                 SQL_DIAG_ROW_COUNT, &rowCount,
+                                 0, NULL));
+  is_num(rowCount, 3);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "DROP TABLE bug11750296");
+
+  return OK;
+}
+
+
+DECLARE_TEST(t_bug25671389)
+{
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+
+  SQLINTEGER connection_id;
+  SQLCHAR buf[255];
+  SQLHSTMT hstmt2;
+
+  SQLRETURN rc = SQL_ERROR;
+  SQLCHAR sqlState[6] = { '\0' };
+  SQLCHAR eMsg[SQL_MAX_MESSAGE_LENGTH] = { '\0' };
+  SQLINTEGER nError = 0;
+  SQLSMALLINT msgLen = 0;
+
+  is(OK == alloc_basic_handles(&henv1, &hdbc1, &hstmt1));
+  /* The bug is repeatable when limit is set and two STMTs exist */
+  ok_con(hdbc1, SQLAllocHandle(SQL_HANDLE_STMT, hdbc1, &hstmt2));
+  ok_stmt(hstmt2, SQLSetStmtAttr(hstmt2, SQL_ATTR_MAX_ROWS, (SQLPOINTER)100, 0));
+
+  ok_sql(hstmt1, "SELECT connection_id()");
+  ok_stmt(hstmt1, SQLFetch(hstmt1));
+  connection_id = my_fetch_int(hstmt1, 1);
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
+
+  /* From another connection, kill the connection created above */
+  sprintf(buf, "KILL %d", connection_id);
+  ok_stmt(hstmt, SQLExecDirect(hstmt, (SQLCHAR *)buf, SQL_NTS));
+
+  /* Now check that the connection killed returns the right SQLSTATE */
+  expect_sql(hstmt2, "SELECT connection_id()", SQL_ERROR);
+
+  rc = SQLGetDiagRec(SQL_HANDLE_STMT, hstmt2, 1, sqlState, &nError, eMsg, sizeof(eMsg), &msgLen);
+  /* Check that the error has been properly reported */
+  is(rc == SQL_SUCCESS);
+  is(nError > 0);
+  is(strlen(sqlState) > 0);
+  is(strlen(eMsg) > 0);
+  is(msgLen > 0);
+
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+  return OK;
+}
+
+
 BEGIN_TESTS
+  ADD_TEST(t_get_diag_all)
 #ifndef NO_DRIVERMANAGER
-  ADD_TEST(t_odbc2_error)
-  ADD_TEST(t_odbc3_error)
-  /* Run twice to test the driver's handling of switching  */
-  ADD_TEST(t_odbc2_error)
+#ifndef USE_IODBC
+  // This test will be re-enabled upon merging with 8.0 sources
+  // ADD_TEST(t_odbc2_error) 
 #endif
+  ADD_TEST(t_odbc3_error)
+#ifndef USE_IODBC
+  // ADD_TEST(t_odbc3_80) TODO: Fix
+#endif
+#endif
+  ADD_TEST(t_bug_28258500)
   ADD_TEST(t_diagrec)
   ADD_TEST(t_warning)
   ADD_TEST(t_bug3456)
   ADD_TEST(t_bug16224)
+  ADD_TEST(t_bug25671389)
+#ifndef USE_IODBC
   ADD_TEST(bind_invalidcol)
+  ADD_TEST(t_handle_err)
+#endif
   ADD_TEST(bind_notenoughparam1)
   ADD_TEST(bind_notenoughparam2)
   ADD_TEST(getdata_need_nullind)
-  ADD_TEST(t_handle_err)
   ADD_TEST(sqlerror)
   ADD_TEST(t_bug27158)
-  ADD_TEST(t_bug13542600)
-  ADD_TEST(t_bug14285620)
+  // ADD_TEST(t_bug13542600) TODO: Fix
+  // ADD_TEST(t_bug14285620) TODO: Fix
+  // ADD_TOFIX(t_bug49466) TODO: Fix
   ADD_TEST(t_passwordexpire)
-  ADD_TEST(t_cleartext_password)
+  // ADD_TEST(t_cleartext_password) TODO: Fix Segfault
+  // ADD_TEST(t_bug11750296) TODO: Fix
 END_TESTS
 
 RUN_TESTS
