@@ -1,26 +1,30 @@
-/*
-  Copyright (c) 2001, 2013, Oracle and/or its affiliates. All rights reserved.
-
-  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-  MySQL Connectors. There are special exceptions to the terms and
-  conditions of the GPLv2 as it is applied to this software, see the
-  FLOSS License Exception
-  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-  
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; version 2 of the License.
-  
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-  for more details.
-  
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+// Copyright (c) 2001, 2018, Oracle and/or its affiliates. All rights reserved. 
+// 
+// This program is free software; you can redistribute it and/or modify 
+// it under the terms of the GNU General Public License, version 2.0, as 
+// published by the Free Software Foundation. 
+// 
+// This program is also distributed with certain software (including 
+// but not limited to OpenSSL) that is licensed under separate terms, 
+// as designated in a particular file or component or in included license 
+// documentation. The authors of MySQL hereby grant you an 
+// additional permission to link the program and your derivative works 
+// with the separately licensed software that they have included with 
+// MySQL. 
+// 
+// Without limiting anything contained in the foregoing, this file, 
+// which is part of <MySQL Product>, is also subject to the 
+// Universal FOSS Exception, version 1.0, a copy of which can be found at 
+// http://oss.oracle.com/licenses/universal-foss-exception. 
+// 
+// This program is distributed in the hope that it will be useful, but 
+// WITHOUT ANY WARRANTY; without even the implied warranty of 
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
+// See the GNU General Public License, version 2.0, for more details. 
+// 
+// You should have received a copy of the GNU General Public License 
+// along with this program; if not, write to the Free Software Foundation, Inc., 
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
 
 
 #include "odbctap.h"
@@ -2256,29 +2260,31 @@ BEGIN_TESTS
   ADD_TEST(t_convert_type)
   ADD_TEST(t_desc_col)
   ADD_TEST(t_convert)
+#ifndef USE_IODBC
   ADD_TEST(t_max_rows)
+  ADD_TEST(t_empty_str_bug)
+  ADD_TEST(tmysql_rowstatus)
+  ADD_TEST(t_bug27544)
+  ADD_TEST(bug6157)
+  ADD_TEST(t_bug30958_wchar)
+  ADD_TEST(t_bug13776)
+#endif
   ADD_TEST(t_multistep)
   ADD_TEST(t_zerolength)
   ADD_TEST(t_cache_bug)
   ADD_TEST(t_non_cache_bug)
-  ADD_TEST(t_empty_str_bug)
   ADD_TEST(t_desccol)
-  ADD_TEST(t_desccolext)
+  // ADD_TEST(t_desccolext) TODO: Fix
   ADD_TEST(t_desccol1)
-  ADD_TEST(t_colattributes)
+  // ADD_TEST(t_colattributes) TODO: Fix
   ADD_TEST(t_exfetch)
-  ADD_TEST(tmysql_rowstatus)
   ADD_TEST(t_true_length)
-  ADD_TEST(t_bug27544)
   ADD_TEST(t_bug16817)
-  ADD_TEST(bug6157)
   ADD_TEST(t_binary_collation)
   ADD_TEST(t_bug29239)
   ADD_TEST(t_bug30958)
   ADD_TEST(t_bug30958_ansi)
-  ADD_TEST(t_bug30958_wchar)
   ADD_TEST(t_bug31246)
-  ADD_TEST(t_bug13776)
   ADD_TEST(t_bug13776_auto)
   ADD_TEST(t_bug28617)
   ADD_TEST(t_bug34429)
