@@ -1,26 +1,30 @@
-/*
-  Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
-
-  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-  MySQL Connectors. There are special exceptions to the terms and
-  conditions of the GPLv2 as it is applied to this software, see the
-  FLOSS License Exception
-  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-  
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; version 2 of the License.
-  
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-  for more details.
-  
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+// Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved. 
+// 
+// This program is free software; you can redistribute it and/or modify 
+// it under the terms of the GNU General Public License, version 2.0, as 
+// published by the Free Software Foundation. 
+// 
+// This program is also distributed with certain software (including 
+// but not limited to OpenSSL) that is licensed under separate terms, 
+// as designated in a particular file or component or in included license 
+// documentation. The authors of MySQL hereby grant you an 
+// additional permission to link the program and your derivative works 
+// with the separately licensed software that they have included with 
+// MySQL. 
+// 
+// Without limiting anything contained in the foregoing, this file, 
+// which is part of <MySQL Product>, is also subject to the 
+// Universal FOSS Exception, version 1.0, a copy of which can be found at 
+// http://oss.oracle.com/licenses/universal-foss-exception. 
+// 
+// This program is distributed in the hope that it will be useful, but 
+// WITHOUT ANY WARRANTY; without even the implied warranty of 
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
+// See the GNU General Public License, version 2.0, for more details. 
+// 
+// You should have received a copy of the GNU General Public License 
+// along with this program; if not, write to the Free Software Foundation, Inc., 
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
 
 
 #include "odbctap.h"
@@ -59,7 +63,7 @@ DECLARE_TEST(t_bug32420)
   ok_stmt(hstmt1, SQLBindCol(hstmt1, 1, SQL_C_LONG, nData, 0, NULL));
   ok_stmt(hstmt1, SQLBindCol(hstmt1, 2, SQL_C_CHAR, szData, sizeof(szData[0]),
                             NULL));
-  ok_stmt(hstmt1, SQLExtendedFetch(hstmt1, SQL_FETCH_NEXT, 0, NULL, 
+  ok_stmt(hstmt1, SQLExtendedFetch(hstmt1, SQL_FETCH_NEXT, 0, NULL,
                                    rgfRowStatus));
 
   is_num(nData[0], 100);
@@ -223,7 +227,7 @@ DECLARE_TEST(t_bug39644)
 
   ok_sql(hstmt, "drop table if exists t_bug39644");
   ok_sql(hstmt, "create table t_bug39644(col1 INT, col2 INT,"\
-	            "col3 BIT, col4 BIT)");
+              "col3 BIT, col4 BIT)");
 
   ok_sql(hstmt, "insert into t_bug39644 VALUES (5, 0, 1, 0)");
 
@@ -327,9 +331,9 @@ DECLARE_TEST(t_bug32821)
       case 1: is_num(b, expected_a[i]); break;
       case 2: is_num(b, expected_b[i]); break;
       }
-      
+
     }
- 
+
     ++i;
   }
 
@@ -398,7 +402,7 @@ DECLARE_TEST(t_bug32684)
   {
     ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_WCHAR, wbuf,
                               20 * sizeof(SQLWCHAR), &wlen));
-    wprintf(L"# data= %s, len=%d\n\n", wbuf, wlen);
+    wprintf(L"# data= %s, len=%d\n\n", wbuf, (int)wlen);
   } while(wlen > 20 * sizeof(SQLWCHAR));
 
   return OK;
@@ -458,7 +462,7 @@ DECLARE_TEST(t_bug55024)
 
 
 /*
-Bug #56677 - SQLNumResultCols() causes the driver to return 
+Bug #56677 - SQLNumResultCols() causes the driver to return
 only first row in the resultset
 */
 DECLARE_TEST(t_bug56677)
@@ -668,7 +672,7 @@ DECLARE_TEST(t_row_status)
   for (i= 0; i<2; ++i)
   {
     printMessage("Row %d, Desc %d, Parameter %d", i+1, arr1[i], arr2[i]);
-    is_num(expectedRow1[i], arr1[i])
+    is_num(expectedRow1[i], arr1[i]);
     is_num(arr1[i], arr2[i]);
   }
 
@@ -678,7 +682,7 @@ DECLARE_TEST(t_row_status)
                                        from b_row_status\
                                        order by i desc", SQL_NTS));
 
-  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_CHAR, res, 5, NULL));  
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_CHAR, res, 5, NULL));
 
   for (i= 0; i<2; ++i)
   {
@@ -687,8 +691,7 @@ DECLARE_TEST(t_row_status)
     {
       printMessage("Set %d Row %d, desc %d, parameter %d", i+1, j+1, arr1[j],
                     arr2[j]);
-      is_num(expectedRow2[i][j], arr1[j])
-
+      is_num(expectedRow2[i][j], arr1[j]);
     }
   }
 
@@ -741,6 +744,45 @@ DECLARE_TEST(t_prefetch)
     return OK;
 }
 
+DECLARE_TEST(t_bug17386788)
+{
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+  SQLINTEGER nidata=7;
+
+  ok_sql(hstmt, "DROP table IF EXISTS b_bug17386788");
+  ok_sql(hstmt, "CREATE table b_bug17386788(i int)");
+
+  ok_sql(hstmt, "insert into b_bug17386788 values(1),(2),(3),(4),(5),(6),(7)");
+
+  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
+                                        NULL, NULL, NULL, "PREFETCH=5"));
+
+  ok_stmt(hstmt1,
+          SQLPrepare(hstmt1,
+                     (SQLCHAR *)"select * from b_bug17386788 where i < ? ",
+                     SQL_NTS)
+          );
+
+
+  ok_stmt(hstmt1,
+          SQLBindParameter(hstmt1,1,SQL_PARAM_INPUT, SQL_C_LONG,SQL_INTEGER,
+                           0,0,&nidata,20,NULL)
+          );
+
+  ok_stmt(hstmt1, SQLExecute(hstmt1));
+
+  is_num(6, myrowcount(hstmt1));
+
+  expect_stmt(hstmt1, SQLMoreResults(hstmt1), SQL_NO_DATA);
+
+  ok_sql(hstmt, "DROP table IF EXISTS b_bug17386788");
+
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+
+  return OK;
+
+}
+
 
 DECLARE_TEST(t_outparams)
 {
@@ -785,7 +827,7 @@ DECLARE_TEST(t_outparams)
 
   ok_stmt(hstmt, SQLNumResultCols(hstmt,&ncol));
   is_num(ncol, 3);
-  
+
   ok_stmt(hstmt, SQLFetch(hstmt));
   is_num(my_fetch_int(hstmt, 1), 300);
   is_num(my_fetch_int(hstmt, 2), 100);
@@ -814,12 +856,12 @@ DECLARE_TEST(t_outparams)
 
 
 /*
-  Bug #11766437: Incorrect increment(increments in multiple of SQLLEN) of 
-  pointer to the length/indicator buffer(last parameter of SQLBindCol), 
-  which gives incorrrect result when SQL_ATTR_ROW_BIND_TYPE is set to 
-  size of data inserted which is not not multiple of 8 on 64 bit 
+  Bug #11766437: Incorrect increment(increments in multiple of SQLLEN) of
+  pointer to the length/indicator buffer(last parameter of SQLBindCol),
+  which gives incorrrect result when SQL_ATTR_ROW_BIND_TYPE is set to
+  size of data inserted which is not not multiple of 8 on 64 bit
   system where sizeof SQLLEN is 8.
-  Tests for data fetched with SQL_ATTR_ROW_BIND_TYPE size set to 
+  Tests for data fetched with SQL_ATTR_ROW_BIND_TYPE size set to
   multiple of 2, binded buffers are checked for proper data fetch.
 */
 DECLARE_TEST(t_bug11766437)
@@ -829,7 +871,7 @@ DECLARE_TEST(t_bug11766437)
   SQLCHAR tbuf[50];
   char *ptr;
   char rows[500]= {0};
-  SQLINTEGER MAX_CHAR_SIZE= 7; /*max size for character name*/ 
+  SQLINTEGER MAX_CHAR_SIZE= 7; /*max size for character name*/
 
   ok_sql(hstmt, "drop table if exists t_bug11766437");
   ok_sql(hstmt, "create table t_bug11766437 (id int not null, "
@@ -839,44 +881,44 @@ DECLARE_TEST(t_bug11766437)
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
 
   ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_ARRAY_SIZE,
-                                (SQLPOINTER)rowcnt, 0));
+                                (SQLPOINTER)((size_t)rowcnt), 0));
 
   /*
-    With same text inserted we change binding orientation 
+    With same text inserted we change binding orientation
     to verify our changes
-  */ 
+  */
   for (incr= 0; incr <= 24; incr += 2)
   {
-    size_t row_size= sizeof(SQLINTEGER) + sizeof(SQLLEN) + 
+    size_t row_size= sizeof(SQLINTEGER) + sizeof(SQLLEN) +
               sizeof(SQLLEN) + MAX_CHAR_SIZE + incr;
 
     /*
-      Set SQL_ATTR_ROW_BIND_TYPE to the size of the data inserted 
-      with multiple of 2 increment 
+      Set SQL_ATTR_ROW_BIND_TYPE to the size of the data inserted
+      with multiple of 2 increment
     */
     ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_BIND_TYPE,
                                   (SQLPOINTER)row_size, 0));
 
     /*
-      Binding all parameters with same buffer to test proper 
+      Binding all parameters with same buffer to test proper
       increment of last parameter of SQLBindCol
     */
     ptr= rows;
-    ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_LONG, 
+    ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_LONG,
       (SQLPOINTER) ptr,
       (SQLLEN) sizeof(SQLINTEGER),
       (SQLLEN *) (ptr + sizeof(SQLINTEGER))));
 
     /*
-      Incrementing pointer position by sizeof(SQLINTEGER) i.e. size of id 
+      Incrementing pointer position by sizeof(SQLINTEGER) i.e. size of id
       and sizeof(SQLLEN) bytes required to store length of id
     */
     ptr += sizeof(SQLINTEGER) + sizeof(SQLLEN);
-    ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_CHAR, 
+    ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_CHAR,
       (SQLPOINTER) ptr,
       (SQLLEN) MAX_CHAR_SIZE,
       (SQLLEN *) (ptr + MAX_CHAR_SIZE)));
-   
+
     ok_sql(hstmt, "select id,name from t_bug11766437 order by id");
 
     ok_stmt(hstmt, SQLFetch(hstmt));
@@ -918,10 +960,241 @@ DECLARE_TEST(t_bug11766437)
 }
 
 
-/* 
-  Bug #17311065: ASSERT FAILURE IN SQLDESCRIBECOL() 
+/**
+  Bind column 0 with SQL_C_VARBOOKMARK data type and retrieve bookmark
+  using FetchOrientation SQL_FETCH_BOOKMARK.
+  Set position in rowset using SQLSetPos and retrieve bookmark for that
+  position using SQLGetData and save that particular bookmark position
+  using SQL_ATTR_FETCH_BOOKMARK_PTR. Now fetch using using
+  FetchOrientation SQL_FETCH_BOOKMARK with retrieve data from that position.
+*/
+DECLARE_TEST(t_varbookmark)
+{
+  SQLLEN len= 0;
+  SQLCHAR abookmark[20];
+  SQLINTEGER outlen;
+  SQLUSMALLINT rowStatus[11];
+  SQLUINTEGER numRowsFetched;
+  SQLINTEGER nData[11];
+  SQLCHAR szData[11][16];
+  SQLCHAR bData[11][10];
+
+  ok_sql(hstmt, "drop table if exists t_bookmark");
+  ok_sql(hstmt, "CREATE TABLE t_bookmark ("\
+                "tt_int INT PRIMARY KEY auto_increment,"\
+                "tt_varchar VARCHAR(128) NOT NULL)");
+  ok_sql(hstmt, "INSERT INTO t_bookmark VALUES "\
+                "(100, 'string 1'),"\
+                "(200, 'string 2'),"\
+                "(300, 'string 3'),"\
+                "(400, 'string 4'),"\
+                "(500, 'string 5'),"\
+                "(600, 'string 6'),"\
+                "(700, 'string 7'),"\
+                "(800, 'string 8'),"\
+                "(900, 'string 9'),"\
+                "(1000, 'string 10'),"\
+                "(1100, 'string 11')");
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_USE_BOOKMARKS,
+                                (SQLPOINTER) SQL_UB_VARIABLE, 0));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_STATUS_PTR,
+                                (SQLPOINTER)rowStatus, 0));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROWS_FETCHED_PTR,
+                                &numRowsFetched, 0));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
+
+
+  ok_stmt(hstmt, SQLSetStmtOption(hstmt, SQL_ROWSET_SIZE, 11));
+
+  ok_sql(hstmt, "select * from t_bookmark order by 1");
+  ok_stmt(hstmt, SQLBindCol(hstmt, 0, SQL_C_VARBOOKMARK, bData,
+                          sizeof(bData[0]), NULL));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_LONG, nData, 0, NULL));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_CHAR, szData, sizeof(szData[0]),
+                            NULL));
+
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_FIRST, 0));
+
+  is_num(nData[0], 100);
+  is_str(szData[0], "string 1", 8);
+  is_num(nData[1], 200);
+  is_str(szData[1], "string 2", 8);
+  is_num(nData[2], 300);
+  is_str(szData[2], "string 3", 8);
+  is_num(nData[3], 400);
+  is_str(szData[3], "string 4", 8);
+
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_BOOKMARK, 0));
+
+  is_num(nData[0], 100);
+  is_str(szData[0], "string 1", 8);
+  is_num(atol(bData[0]), 1);
+  is_num(nData[1], 200);
+  is_str(szData[1], "string 2", 8);
+  is_num(atol(bData[1]), 2);
+  is_num(nData[2], 300);
+  is_str(szData[2], "string 3", 8);
+  is_num(atol(bData[2]), 3);
+  is_num(nData[3], 400);
+  is_str(szData[3], "string 4", 8);
+  is_num(atol(bData[3]), 4);
+
+  is_num(nData[4], 500);
+  is_str(szData[4], "string 5", 8);
+  is_num(atol(bData[4]), 5);
+  is_num(nData[5], 600);
+  is_str(szData[5], "string 6", 8);
+  is_num(atol(bData[5]), 6);
+  is_num(nData[6], 700);
+  is_str(szData[6], "string 7", 8);
+  is_num(atol(bData[6]), 7);
+  is_num(nData[7], 800);
+  is_str(szData[7], "string 8", 8);
+  is_num(atol(bData[7]), 8);
+
+  is_num(nData[8], 900);
+  is_str(szData[8], "string 9", 8);
+  is_num(atol(bData[8]), 9);
+  is_num(nData[9], 1000);
+  is_str(szData[9], "string 10", 8);
+  is_num(atol(bData[9]), 10);
+  is_num(nData[10], 1100);
+  is_str(szData[10], "string 11", 8);
+  is_num(atol(bData[10]), 11);
+
+  ok_stmt(hstmt, SQLSetPos(hstmt, 2, SQL_POSITION, SQL_LOCK_NO_CHANGE));
+  ok_stmt(hstmt, SQLGetData(hstmt, 0, SQL_C_VARBOOKMARK, abookmark, 255, &outlen));
+
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_FETCH_BOOKMARK_PTR,
+                         (SQLPOINTER) abookmark, 0));
+
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_BOOKMARK, 0));
+
+  is_num(nData[0], 200);
+  is_str(szData[0], "string 2", 8);
+  is_num(atol(bData[0]), 2);
+  is_num(nData[1], 300);
+  is_str(szData[1], "string 3", 8);
+  is_num(atol(bData[1]), 3);
+  is_num(nData[2], 400);
+  is_str(szData[2], "string 4", 8);
+  is_num(atol(bData[2]), 4);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "drop table if exists t_bookmark");
+
+  return OK;
+}
+
+
+/**
+  Bind column 0 with SQL_C_BOOKMARK data type and retrieve bookmark
+  using FetchOrientation SQL_FETCH_BOOKMARK.
+  Set position in rowset using SQLSetPos and retrieve bookmark for that
+  position using SQLGetData and save that particular bookmark position
+  using SQL_ATTR_FETCH_BOOKMARK_PTR. Now fetch using using
+  FetchOrientation SQL_FETCH_BOOKMARK with retrieve data from that position.
+*/
+#ifdef _UNIX_
+DECLARE_TEST(t_bookmark)
+{
+  SQLLEN len= 0;
+  SQLINTEGER abookmark[4];
+  SQLINTEGER outlen;
+  SQLUSMALLINT rowStatus[4];
+  SQLUINTEGER numRowsFetched;
+  SQLINTEGER nData[4];
+  SQLCHAR szData[4][16];
+  SQLINTEGER bData[4];
+
+  ok_sql(hstmt, "drop table if exists t_bookmark");
+  ok_sql(hstmt, "CREATE TABLE t_bookmark ("\
+                "tt_int INT PRIMARY KEY auto_increment,"\
+                "tt_varchar VARCHAR(128) NOT NULL)");
+  ok_sql(hstmt, "INSERT INTO t_bookmark VALUES "\
+                "(100, 'string 1'),"\
+                "(200, 'string 2'),"\
+                "(300, 'string 3'),"\
+                "(400, 'string 4')");
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_USE_BOOKMARKS,
+                                (SQLPOINTER) SQL_UB_VARIABLE, 0));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROW_STATUS_PTR,
+                                (SQLPOINTER)rowStatus, 0));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_ROWS_FETCHED_PTR,
+                                &numRowsFetched, 0));
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_CURSOR_TYPE,
+                                (SQLPOINTER)SQL_CURSOR_STATIC, 0));
+
+
+  ok_stmt(hstmt, SQLSetStmtOption(hstmt, SQL_ROWSET_SIZE, 4));
+
+  ok_sql(hstmt, "select * from t_bookmark order by 1");
+  ok_stmt(hstmt, SQLBindCol(hstmt, 0, SQL_C_BOOKMARK, bData,
+                          sizeof(bData[0]), NULL));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 1, SQL_C_LONG, nData, 0, NULL));
+  ok_stmt(hstmt, SQLBindCol(hstmt, 2, SQL_C_CHAR, szData, sizeof(szData[0]),
+                            NULL));
+
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_FIRST, 0));
+
+  is_num(nData[0], 100);
+  is_str(szData[0], "string 1", 8);
+  is_num(nData[1], 200);
+  is_str(szData[1], "string 2", 8);
+  is_num(nData[2], 300);
+  is_str(szData[2], "string 3", 8);
+  is_num(nData[3], 400);
+  is_str(szData[3], "string 4", 8);
+
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_BOOKMARK, 0));
+
+  is_num(nData[0], 100);
+  is_str(szData[0], "string 1", 8);
+  is_num(bData[0], 1);
+  is_num(nData[1], 200);
+  is_str(szData[1], "string 2", 8);
+  is_num(bData[1], 2);
+  is_num(nData[2], 300);
+  is_str(szData[2], "string 3", 8);
+  is_num(bData[2], 3);
+  is_num(nData[3], 400);
+  is_str(szData[3], "string 4", 8);
+  is_num(bData[3], 4);
+
+  ok_stmt(hstmt, SQLSetPos(hstmt, 3, SQL_POSITION, SQL_LOCK_NO_CHANGE));
+  ok_stmt(hstmt, SQLGetData(hstmt, 0, SQL_C_BOOKMARK, abookmark, 255, &outlen));
+
+  ok_stmt(hstmt, SQLSetStmtAttr(hstmt, SQL_ATTR_FETCH_BOOKMARK_PTR,
+                         (SQLPOINTER) abookmark, 0));
+
+  ok_stmt(hstmt, SQLFetchScroll(hstmt, SQL_FETCH_BOOKMARK, 0));
+
+  is_num(nData[0], 300);
+  is_str(szData[0], "string 3", 8);
+  is_num(bData[0], 3);
+  is_num(nData[1], 400);
+  is_str(szData[1], "string 4", 8);
+  is_num(bData[1], 4);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_sql(hstmt, "drop table if exists t_bookmark");
+
+  return OK;
+}
+#endif
+
+
+/*
+  Bug #17311065: ASSERT FAILURE IN SQLDESCRIBECOL()
                  IF THE COLUMN NUMBER GIVEN IS LARGE
-  Assert failure in case of prepared statments and SQLDESCRIBECOL() called 
+  Assert failure in case of prepared statments and SQLDESCRIBECOL() called
   with column number given larger then actual parameter marker in query.
 */
 DECLARE_TEST(t_bug17311065)
@@ -956,6 +1229,149 @@ DECLARE_TEST(t_bug17311065)
 }
 
 
+#define RESTORE_SERVER_LOGGING ok_sql(hstmt, "SET GLOBAL log_output=@old_log_output, general_log=@old_general_log")
+
+#define ok_stmt2(S,C) do { \
+  SQLRETURN rc2 = (C); \
+  if (rc2 != SQL_SUCCESS && rc2 != SQL_SUCCESS_WITH_INFO) \
+  { \
+    RESTORE_SERVER_LOGGING; \
+    ok_stmt(S, rc2); \
+  } \
+} while (0)
+
+#define is_num2(a, b) \
+do { \
+  long long a1= (a), a2= (b); \
+  if (a1 != a2) { \
+    RESTORE_SERVER_LOGGING; \
+    printf("# %s (%lld) != %lld in %s on line %d\n", \
+           #a, a1, a2, __FILE__, __LINE__); \
+    return FAIL; \
+  } \
+} while (0)
+
+
+DECLARE_TEST(t_prefetch_bug)
+{
+    int param_1 = 1, param_2 = 20, i = 0;
+    long con_id = 0;
+
+    const char *queries[] = {
+      "select 'Q-001', id from b_prefecth WHERE id > ? ORDER BY id",
+      "select 'Q-002', id from b_prefecth WHERE id > ? ORDER BY id LIMIT 7",
+      "select 'Q-003', id from b_prefecth WHERE id > ? ORDER BY id LIMIT 5, 3",
+      "select 'Q-004', id from b_prefecth WHERE id > ? ORDER BY id LIMIT 5, 100",
+      "select 'Q-005', id from b_prefecth WHERE id > ? ORDER BY id FOR UPDATE",
+      "select 'Q-006', id from b_prefecth WHERE id > ? ORDER BY id LOCK IN SHARE MODE",
+      "select 'Q-007', id from b_prefecth WHERE id > ? ORDER BY id LIMIT 2, 20 FOR UPDATE",
+      "select 'Q-008', id from b_prefecth WHERE id > ? ORDER BY id LIMIT 2, 20 LOCK IN SHARE MODE",
+      "select 'Q-009', id from b_prefecth WHERE id > ? ORDER BY id LIMIT 2, ? FOR UPDATE",
+    };
+
+    const char *check_for[] = {
+      "",       //"select 'Q-001' ... ORDER BY id",
+      "",       //"select 'Q-002' ... ORDER BY id LIMIT 7",
+      "",       //"select 'Q-003' ... ORDER BY id LIMIT 5, 3",
+      "",       //"select 'Q-004' ... ORDER BY id LIMIT 5, 100",
+      // "select 'Q-005' ... ORDER BY id FOR UPDATE"
+      " AND argument LIKE '%%FOR UPDATE%%'",
+      // "select 'Q-006' ... ORDER BY id LOCK IN SHARE MODE",
+      " AND argument LIKE '%%LOCK IN SHARE MODE%%'",
+      // "select 'Q-007' ... ORDER BY id FOR UPDATE"
+      " AND argument LIKE '%%FOR UPDATE%%'",
+      // "select 'Q-008' ... ORDER BY id LOCK IN SHARE MODE",
+      " AND argument LIKE '%%LOCK IN SHARE MODE%%'",
+      // "select 'Q-009' ... ORDER BY id FOR UPDATE"
+      " AND argument LIKE '%%FOR UPDATE%%'"
+    };
+
+    int expected_num[]     = {15, 7, 3, 10, 15, 15, 13, 13, 13};
+    int expected_queries[] = { 4, 2, 1,  3,  4,  4,  4,  4, 4 };
+    char check_query[512];
+    int query_rows = 0;
+
+    DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+
+    is(OK == alloc_basic_handles_with_opt(
+             &henv1, &hdbc1, &hstmt1, NULL,
+             NULL, NULL, NULL, "PREFETCH=5;NO_SSPS=1"));
+
+    ok_sql(hstmt, "DROP table IF EXISTS b_prefecth");
+    ok_sql(hstmt, "CREATE table b_prefecth(id int primary key)");
+
+    ok_sql(hstmt, "insert into b_prefecth values(1),(2),(3),(4)," \
+          "(5),(6),(7),(8),(9),(10),(11),(12),(13),(14),(15),(16)");
+
+    // Save the old values for
+    ok_sql(hstmt1, "SELECT CONNECTION_ID()");
+    ok_stmt(hstmt1, SQLFetch(hstmt1));
+    ok_stmt(hstmt1, SQLGetData(hstmt1, 1, SQL_C_LONG, &con_id, 0, NULL));
+    printf("\nConnection ID: %d",con_id);
+    ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
+
+    ok_sql(hstmt, "SET @old_log_output=@@log_output, @old_general_log=@@general_log");
+    ok_sql(hstmt, "SET GLOBAL log_output='TABLE', general_log=1");
+
+    for (i = 0; i < 9; ++i)
+    {
+      ok_stmt2(hstmt1, SQLPrepare(hstmt1, (SQLCHAR*)queries[i], SQL_NTS));
+      ok_stmt2(hstmt1, SQLBindParameter(hstmt1, 1, SQL_PARAM_INPUT,
+        SQL_C_LONG, SQL_INTEGER, 0, 0, &param_1, 0, NULL));
+
+      if (i == 8) // Bind 2nd param for LIMIT
+        ok_stmt2(hstmt1, SQLBindParameter(hstmt1, 2, SQL_PARAM_INPUT,
+        SQL_C_LONG, SQL_INTEGER, 0, 0, &param_2, 0, NULL));
+
+      ok_stmt2(hstmt1, SQLExecute(hstmt1));
+      is_num2(expected_num[i], my_print_non_format_result(hstmt1));
+      sprintf(check_query,
+                "SELECT CAST(argument as CHAR(128)) arg FROM " \
+                "mysql.general_log WHERE thread_id = %d AND " \
+                "argument LIKE '%% LIMIT %%' AND " \
+                "argument LIKE '%%Q-%03d%%' %s",
+                con_id, (i + 1), check_for[i]);
+
+      ok_stmt2(hstmt, SQLExecDirect(hstmt, (SQLCHAR*)check_query, SQL_NTS));
+      printf("\nLOGGED QUERIES:\n");
+      query_rows = my_print_non_format_result(hstmt);
+
+      // Check if the logging actually happened
+      if (query_rows == 0)
+      {
+        int logged_rows = 0;
+        sprintf(check_query, "SELECT CAST(argument as CHAR(128)) arg FROM " \
+                "mysql.general_log WHERE thread_id = %d", con_id);
+        ok_stmt2(hstmt, SQLExecDirect(hstmt, (SQLCHAR*)check_query, SQL_NTS));
+        logged_rows = my_print_non_format_result(hstmt);
+        RESTORE_SERVER_LOGGING;
+        if (logged_rows > 0)
+        {
+          printf("\nUnexpected logged rows!");
+          return FAIL;
+        }
+        else
+        {
+          printf("\nWarning: General query log table is empty");
+          return OK;
+        }
+      }
+
+      if (expected_queries[i] < query_rows ||
+          (expected_queries[i] - 1) > query_rows )
+      {
+        RESTORE_SERVER_LOGGING;
+        return FAIL;
+      }
+    }
+
+    RESTORE_SERVER_LOGGING;
+    free_basic_handles(&henv1, &hdbc1, &hstmt1);
+    ok_sql(hstmt, "DROP table IF EXISTS b_prefecth");
+
+    return OK;
+}
+
 BEGIN_TESTS
   ADD_TEST(t_bug32420)
   ADD_TEST(t_bug34575)
@@ -963,18 +1379,26 @@ BEGIN_TESTS
   ADD_TEST(t_bug36069)
   ADD_TEST(t_bug41942)
   ADD_TEST(t_bug39644)
-  ADD_TEST(t_bug32821)
+  // ADD_TEST(t_bug32821) TODO: Fix
   ADD_TEST(t_bug34271)
   ADD_TEST(t_bug32684)
   ADD_TEST(t_bug55024)
   ADD_TEST(t_bug56677)
   ADD_TEST(t_desccol_before_exec)
   ADD_TEST(t_bug62657)
+#ifndef USE_IODBC
   ADD_TEST(t_row_status)
+#endif
   ADD_TEST(t_prefetch)
+  ADD_TEST(t_bug17386788)
   ADD_TOFIX(t_outparams)
-  ADD_TEST(t_bug11766437)
+  // ADD_TEST(t_bug11766437) TODO: Fix
+  // ADD_TEST(t_varbookmark) TODO: Fix
+#ifdef _UNIX_
+  // ADD_TEST(t_bookmark) TODO: Fix
+#endif
   ADD_TEST(t_bug17311065)
+  ADD_TEST(t_prefetch_bug)
 END_TESTS
 
 
