@@ -1,29 +1,59 @@
-/*
-  Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
-
-  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
-  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-  MySQL Connectors. There are special exceptions to the terms and
-  conditions of the GPLv2 as it is applied to this software, see the
-  FLOSS License Exception
-  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-  
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; version 2 of the License.
-  
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
-  for more details.
-  
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 #include "odbctap.h"
 
+DECLARE_TEST(t_text_types)
+{
+  char buf[1024] = {0};
+  SQLSMALLINT out_len = 0;
+  int i = 0;
+
+  ok_sql(hstmt, "DROP TABLE IF EXISTS t_text_types");
+  ok_sql(hstmt, "CREATE TABLE t_text_types (col1 integer, "
+                "col2 text, col3 longtext, col4 mediumtext, "
+                "col5 tinytext)");
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "SELECT * FROM t_text_types");
+  for (i = 1; i < 6; i++)
+  {
+    ok_stmt(hstmt, SQLColAttribute(hstmt, i, SQL_DESC_NAME, buf, sizeof(buf),
+                                   &out_len, NULL));
+    printf("[COL: %s]", buf);
+    ok_stmt(hstmt, SQLColAttribute(hstmt, i, SQL_DESC_TYPE_NAME, buf, sizeof(buf),
+                                   &out_len, NULL));
+    printf("[TYPE NAME: %s]\n", buf);
+  }
+  return OK;
+}
+
 
 DECLARE_TEST(t_longlong1)
 {
@@ -808,9 +838,9 @@ int sqlnum_test_from_str(SQLHANDLE hstmt,
   ok_desc(ard, SQLSetDescField(ard, 1, SQL_DESC_TYPE,
                                (SQLPOINTER) SQL_C_NUMERIC, SQL_IS_INTEGER));
   ok_desc(ard, SQLSetDescField(ard, 1, SQL_DESC_PRECISION,
-                               (SQLPOINTER)(SQLINTEGER) prec, SQL_IS_INTEGER));
+                               (SQLPOINTER)(size_t)prec, SQL_IS_INTEGER));
   ok_desc(ard, SQLSetDescField(ard, 1, SQL_DESC_SCALE,
-                               (SQLPOINTER)(SQLINTEGER) scale, SQL_IS_INTEGER));
+                               (SQLPOINTER)(size_t)scale, SQL_IS_INTEGER));
   ok_desc(ard, SQLSetDescField(ard, 1, SQL_DESC_DATA_PTR,
                                sqlnum, SQL_IS_POINTER));
 
@@ -1062,7 +1092,7 @@ DECLARE_TEST(t_bug31220)
   expect_stmt(hstmt, SQLFetch(hstmt), SQL_ERROR);
   is(check_sqlstate(hstmt, "07006") == OK);
   is_num(outlen, 999);
-  return OK;  
+  return OK;
 }
 
 
@@ -1080,9 +1110,9 @@ DECLARE_TEST(t_bug29402)
   const SQLCHAR *expected= "\x80""100";
 
   is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
-                                        NULL, NULL, NULL, 
+                                        NULL, NULL, NULL,
                                         "NO_BINARY_RESULT=1;CHARSET=CP1250"));
-  
+
   ok_stmt(hstmt1, SQLExecDirect(hstmt1, "SELECT CONCAT(_cp1250 0x80, 100) concated", SQL_NTS));
 
   ok_stmt(hstmt1, SQLDescribeCol(hstmt1, 1, column_name, sizeof(column_name),
@@ -1137,12 +1167,13 @@ DECLARE_TEST(t_bug29402)
 
 
 /*
-  Bug #67793 - MySQL ODBC drivers incorrectly returns TIME columns, where 
+  Bug #67793 - MySQL ODBC drivers incorrectly returns TIME columns, where
   value > '99:59:59'
 */
 DECLARE_TEST(t_bug67793)
 {
   SQL_TIME_STRUCT sts;
+  SQL_INTERVAL_STRUCT h2s;
   SQLLEN outlen= 0;
 
   /* make sure we have reset everything to zero */
@@ -1154,18 +1185,18 @@ DECLARE_TEST(t_bug67793)
   ok_sql(hstmt, "SELECT '123456789:45:67', '512:512:512', '20::75:23:10'");
   ok_stmt(hstmt, SQLFetch(hstmt));
 
-  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_TIME, &sts, sizeof(sts), &outlen));
-  is_num(outlen, sizeof(sts));
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_INTERVAL_HOUR_TO_SECOND, &h2s, sizeof(h2s), &outlen));
+  is_num(outlen, sizeof(h2s));
   /* hour cannot go out of unsigned smallint range */
-  is_num(sts.hour, 65535);
-  is_num(sts.minute, 46);
-  is_num(sts.second, 7);
+  is_num(h2s.intval.day_second.hour, 65535);
+  is_num(h2s.intval.day_second.minute, 46);
+  is_num(h2s.intval.day_second.second, 7);
 
-  ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_TIME, &sts, sizeof(sts), &outlen));
-  is_num(outlen, sizeof(sts));
-  is_num(sts.hour, 520);
-  is_num(sts.minute, 40);
-  is_num(sts.second, 32);
+  ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_INTERVAL_HOUR_TO_SECOND, &h2s, sizeof(h2s), &outlen));
+  is_num(outlen, sizeof(h2s));
+  is_num(h2s.intval.day_second.hour, 520);
+  is_num(h2s.intval.day_second.minute, 40);
+  is_num(h2s.intval.day_second.second, 32);
 
   ok_stmt(hstmt, SQLGetData(hstmt, 3, SQL_TIME, &sts, sizeof(sts), &outlen));
   is_num(outlen, sizeof(sts));
@@ -1174,7 +1205,7 @@ DECLARE_TEST(t_bug67793)
   is_num(sts.second, 15);
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
-  return OK;  
+  return OK;
 }
 
 
@@ -1213,27 +1244,30 @@ DECLARE_TEST(t_bug69545)
   }
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
-  return OK;  
+  return OK;
 }
 
 
 BEGIN_TESTS
+  ADD_TEST(t_text_types)
   ADD_TEST(t_longlong1)
   ADD_TEST(t_decimal)
   ADD_TEST(t_bigint)
+#ifndef USE_IODBC
   ADD_TEST(t_enumset)
+  ADD_TEST(t_bug27862_1)
+  // ADD_TODO(t_bug27862_2) TODO: Fix
+  ADD_TEST(binary_suffix)
+  ADD_TEST(t_sqlnum_msdn)
+  ADD_TEST(t_sqlnum_from_str)
+#endif
   ADD_TEST(t_bug16917)
   ADD_TEST(t_bug16235)
-  ADD_TEST(t_bug27862_1)
-  ADD_TODO(t_bug27862_2)
   ADD_TEST(decimal_scale)
-  ADD_TEST(binary_suffix)
   ADD_TEST(float_scale)
   ADD_TEST(bit)
   ADD_TEST(t_bug32171)
-  ADD_TEST_UNICODE(sqlwchar)
-  ADD_TEST(t_sqlnum_msdn)
-  ADD_TEST(t_sqlnum_from_str)
+  // ADD_TEST_UNICODE(sqlwchar) TODO: Fix
   ADD_TEST(t_bindsqlnum_basic)
   ADD_TEST(t_sqlnum_to_str)
   ADD_TEST(t_bug31220)
